#define zleptons_cxx
#include "zleptons.h"
#include <TH2.h>
#include <TStyle.h>
#include <TCanvas.h>
#include <TH1.h>
#include <iostream>

#include "ZZhelper.h"  // functions you need to check if the events is leptonic, to move randomly the x,y,z of the tracks etc
 
using namespace std;

void zleptons::Loop()
{
//   In a ROOT session, you can do:
//      Root > .L zleptons.C
//      Root > zleptons t
//      Root > t.GetEntry(12); // Fill t data members with entry number 12
//      Root > t.Show();       // Show values of entry 12
//      Root > t.Show(16);     // Read and show values of entry 16
//      Root > t.Loop();       // Loop on all entries
//

//     This is the loop skeleton where:
//    jentry is the global entry number in the chain
//    ientry is the entry number in the current Tree
//  Note that the argument to GetEntry must be:
//    jentry for TChain::GetEntry
//    ientry for TTree::GetEntry and TBranch::GetEntry
//
//       To read only selected branches, Insert statements like:
// METHOD1:
//    fChain->SetBranchStatus("*",0);  // disable all branches
//    fChain->SetBranchStatus("branchname",1);  // activate branchname
// METHOD2: replace line
//    fChain->GetEntry(jentry);       //read all branches
//by  b_branchname->GetEntry(ientry); //read only this branch
   if (fChain == 0) return;


 
   // ===> Idea:
   // ----------
   // We will take the pp vertex (mc_vz[0] for the 1st incomming particle) of the first event (even-numbered event:  
   // call this event A, and the p vertex = zpp_A) 
   // and also the mc_vz of the two leptons and their sintheta (mc_vz[2] and mc_sintheta[2], and also mc_vz[3] nd mc_sintheta[3]),
   // let's call them lepton1 and lepton2.
   // Then, keep also the equivalent info from the next event, witht he pp vertex called zpp_B,  
   // (which simulates the case we have two independent pp collisions 
   // in the bunch crossing, each collision giving a Z->2l event).
   // When both of these sets of information is in our hands, check how often we have all 4 leptons
   // consisten with the first vertex: 
   //  abs((zlep1-zpp_A)*sintheta1)<0.5 &&   abs((zlep2-zpp_A)*sintheta2)<0.5  && 
   //  && abs((zlep3-zpp_A)*sintheta3)<0.5  abs((zlep4-zpp_A)*sintheta4)<0.5
   // or with the 2nd vertex zpp_B.
   // --> This way, a very small subset of the these two-event pairs will survive. 
   // 
   // ====> So, use this trick to find the precentage surviving.
   //-----------------------------------------------------------


   Long64_t nentries = fChain->GetEntriesFast();

   // Since I need pairs of events to work (one event from one entry, 
   // and another event from the next entry) I need an even number of entries
   if (nentries%2 != 0) nentries = nentries -1;

   //to test in a small sub-sample:
   // nentries = 1000;


   TH1F *h_zpp_A = new TH1F("h_zpp_A","h_zpp_A",150,0,150);
   
   double zpp_A, zpp_B;
   TLorentzVector tlep1, tlep2, tlep3, tlep4; // reused object
   vector<double> lep_vz;           // will be needed later: to be cleared at start of each event
   vector<double> lep_sintheta;     // as above
   vector<double> lep_pt;
   vector<int> lepIDs;              // as above
   vector<TLorentzVector> tleptons; // will be needed later: to be cleared at start of each event

   // couners:
   int nPairs[3];
   int nPairs_fiducialZZ[3];
   int nPairs_selectedZZ[3];
   for (int i=0; i<3; i++) { nPairs[i]=0; nPairs_fiducialZZ[i]=0; nPairs_selectedZZ[i]=0; }


   Long64_t nbytes = 0, nb = 0;
   for (Long64_t jentry=0; jentry<nentries-1;jentry++) {
      Long64_t ientry = LoadTree(jentry);
      if (ientry < 0) break;
      nb = fChain->GetEntry(jentry);   nbytes += nb;
      // if (Cut(ientry) < 0) continue;
      

      if (jentry%2 == 0) { // even-numbered event  = the first event in the pair
	cout << " checking entries:  " << jentry ;
	zpp_A = mc_vz->at(0); // 0 and 1 are the two incomming partons
	lep_vz.push_back(mc_vz->at(2));  // the leptons are the particles 2 and 3 in the mc_ vectors
	lep_vz.push_back(mc_vz->at(3));
	lep_sintheta.push_back(mc_sintheta->at(2));
	lep_sintheta.push_back(mc_sintheta->at(3));
        lepIDs.push_back(mc_pid->at(2));
        lepIDs.push_back(mc_pid->at(3));
        tlep1.SetPxPyPzE(mc_px->at(2), mc_py->at(2), mc_pz->at(2), mc_E->at(2) );
        tlep2.SetPxPyPzE(mc_px->at(3), mc_py->at(3), mc_pz->at(3), mc_E->at(3) );
        lep_pt.push_back( tlep1.Pt() );
        lep_pt.push_back( tlep2.Pt() );
        tleptons.push_back(tlep1);
        tleptons.push_back(tlep2);
        
      } else {            // odd-numbered event = the second event in the pair
	cout << " and " << jentry << endl;
	zpp_B = mc_vz->at(0);
	lep_vz.push_back(mc_vz->at(2));
	lep_vz.push_back(mc_vz->at(3));
	lep_sintheta.push_back(mc_sintheta->at(2));
	lep_sintheta.push_back(mc_sintheta->at(3));
        lepIDs.push_back(mc_pid->at(2));
        lepIDs.push_back(mc_pid->at(3));
        tlep3.SetPxPyPzE(mc_px->at(2), mc_py->at(2), mc_pz->at(2), mc_E->at(2) );
        tlep4.SetPxPyPzE(mc_px->at(3), mc_py->at(3), mc_pz->at(3), mc_E->at(3) );
        lep_pt.push_back( tlep3.Pt() );
        lep_pt.push_back( tlep4.Pt() );
        tleptons.push_back(tlep3);
        tleptons.push_back(tlep4);

	int decayChannel = ZZchannel(lepIDs);

	nPairs[decayChannel]++;
        if (  isLeptonicZZ(lepIDs) 
	      && isOnShellZZ( (tlep1 + tlep2).M() , (tlep3 + tlep4).M() )
	      && isFiducialZZ(tleptons, lepIDs) ) {
	  nPairs_fiducialZZ[decayChannel]++;
	  if ( isConsistentWithSameVertex(zpp_A, zpp_B, lep_vz, lep_sintheta, lep_pt, true) ) nPairs_selectedZZ[decayChannel]++;
        }

        // clear to be ready to fill info for the 4leptons from the next couple of events
        lep_vz.clear();         // clear to be ready to fill info for the 4leptons from the next couple of events
        lep_sintheta.clear();
        lep_pt.clear();
        lepIDs.clear(); 
        tleptons.clear();

      }


   } // end of loop over entries

   cout << " ********************** SUMMARY  ***************************" << endl;
   for ( int i=0; i<3; i++) {
     cout << "*** Channel # " << ZZtype(i) << " (" << ZZname( ZZtype(i) ) << ") : " << endl;
     cout << "                  All pairs of Z events: " << nPairs[i] 
          << " , Fiducial: " << nPairs_fiducialZZ[i] << " , with all 4 leptons consistent with same vertex: " 
	  << nPairs_selectedZZ[i] << " (" << 100.0*nPairs_selectedZZ[i]/nPairs[i] << " \% )  of all." << endl;
   }

} // end Loop()
